// Fragment Shader

#version 330 core

layout(location = 0) out vec4 final_color;
layout(location = 1) out vec4 edgeinfo_out;


uniform sampler2D color_texture;
uniform sampler2DShadow shadow_map;

//Global alpha
uniform float alpha;

//From vertex shader
in vec3 N; //normal
in vec3 L; //to light
in vec3 E; //to eye

//Contour stuff
in vec4 edgeinfo;
//Shadow stuff
in vec4 SM_tex_coord;

//Color and Material
in vec2 tex;
uniform float ka; //ambient const.
uniform vec3 ambientColor; 
uniform float kd; //diffuse const.
uniform vec3 diffuseColor;
uniform float ks; //specular const.
uniform vec3 specularColor;
uniform uint  phongPower;

//Light
//vec3 lightColor = vec3(1,1,1);

//Parts of final color
vec3 ambient;
vec3 diffuse;
vec3 specular;


void main(void)
{
	
	vec3 texcolor = texture2D ( color_texture, tex ).rgb;

	//Ambient
	ambient = ka * ambientColor * texcolor;

	//Diffuse (Lambert)
	float cosTheta = clamp( dot(N, L), 0,1);
	diffuse = kd * diffuseColor *  texcolor * cosTheta;

	//Specular (Phong)
	vec3 R = - reflect(L, N); //minus, weil anders als in cg folien berechnet
	float cosSigma = clamp( dot(E, R), 0,1); 
	specular = ks * specularColor * texcolor * pow(cosSigma,phongPower);

	//Added
	final_color = vec4(ambient + diffuse + specular, 1.0 * alpha);
	//final_color = vec4(1,1,1,1);


	//Contour-Info
	edgeinfo_out = edgeinfo;

	/*
	//Shadow
	//--perspective division (homogenisieren)
	vec3 tex_coords = SM_tex_coord.xyz/SM_tex_coord.w;
	//--depth value
	float depth = texture(shadow_map, tex_coords.xy).r;
	//--compare
	float inShadow = (depth > tex_coords.z) ? 1.0 : 0.3;
		//activate for shadow mapping
	final_color.xyz = final_color.xyz*inShadow;
	*/
	//GPU SHADOW
	float shadow = textureProj(shadow_map, SM_tex_coord);
	if (shadow == 0 ) {
	final_color.xyz = final_color.xyz*0.3;
		
	}
}
